<?php
/**
 * Plugin pobiera punkty odbioru
 *
 * Smarty plugin
 * @package Smarty
 * @subpackage plugins
 */

require_once ('Cache/Lite.php');

function smarty_function_punkty_odbioru($params, &$smarty){

    if ( !defined('PUNKT_ODBIORU_SP_DOST_ID') || !(PUNKT_ODBIORU_SP_DOST_ID > 0) ) {
        return;
    }
    if ( !defined('PUNKT_ODBIORU_ADRES_ID') || !(PUNKT_ODBIORU_ADRES_ID > 0) ) {
        return;
    }

    $sposoby_dostawy = $smarty->get_template_vars('sposoby_dostawy');
    if( empty($sposoby_dostawy) || !isset($sposoby_dostawy[PUNKT_ODBIORU_SP_DOST_ID]) ){
        return;
    }

    $punkty_odbioru = array();
    $dbh = db_connect1();

    $miejscowosc = empty($params['miejscowosc']) ? '' : trim($params['miejscowosc']);
    $kod = empty($params['kod']) ? '' : trim($params['kod']);

    $Cache_Lite = new Cache_Lite(array(
        'caching' => !empty($params['cache']),
        'cacheDir' => TEMP_DIR . '/cache/',
        'lifeTime' => empty($params['cacheTime']) ? 43200 : $params['cacheTime'],
    ));

    $cache_id = 'smarty_function_punkty_odbioru('.$miejscowosc.', '.$kod.')';

    if (  $data = $Cache_Lite->get($cache_id, 'default', is_resource($dbh) )  ) {
        $punkty_odbioru = unserialize($data);
        echo 'from cache';
    } else {
        $where = $tables = array();

        $tables = array(
            'adres' => 'R3_CONTACT_ADDRESSES adres',

            'contact' => array(
                'table' => 'R3_CONTACTS contact',
                'join' => 'join',
                'on' => 'adres.contact_id = contact.id'
            ),

            'crm' => array(
                'table' => 'R3_CONTACTS_CRM crm',
                'join' => 'left',
                'on' => 'contact.id = crm.contact_id'
            ),

            'desc' => array(
                'table' => 'R3_DESCRIPTIONS opis',
                'join' => 'left',
                'on' => 'contact.description_id = opis.id '
            ),
        );

        $where = array(
            'type' => 'adres.type_id = ' . PUNKT_ODBIORU_ADRES_ID,
            'contact_id' => 'adres.contact_id > 0 ',
        );

        if ( $miejscowosc ) {
            $where['place'] = 'adres.PLACE containing (' .addquotes($miejscowosc). ")";
        }

        if ( $kod && preg_match( '/^[0-9]{2}\-[0-9]{3}$/', $kod ) ) {
            $where['zipcode'] = 'adres.ZIPCODE = \'' . addquotes($kod) . "'";
        }

        $where['order_by'] = 'adres.PLACE, contact.short_name';

        $fields = array( 'adres.*' );
        $fields [] = 'contact.id as contact_id, contact.short_name, contact.full_name, contact.taxid ';
        $fields [] = 'crm.logo_id';
        $fields [] = 'opis.content description';

        $query = make_select( $tables, implode(',', $fields), $where );

        $db = new RaksDefaultClass();
        if( $res = $db -> query( $query ) ){
            while ( $row = ibase_fetch_assoc($res) ) {
                $row = array_change_key_case($row, CASE_LOWER);
                win2iso_r($row);
                $punkty_odbioru[$row['id']] = $row;
            }
            ibase_free_result($res);
        }

        $Cache_Lite->save(serialize($punkty_odbioru));
    }
    $punkty_odbioru_opcje = array();

    foreach ( $punkty_odbioru as $punkt ) {
        $punkt = array_map('trim', $punkt);

        $punkty_odbioru_opcje[$punkt['id']] = $punkt['place'] . ' ' . $punkt['zipcode'] . ', '  .
            $punkt['street'] . ' ' . $punkt['building_number'] . ( empty($punkt['appartment_number']) ? '' : ' ' . $punkt['appartment_number'] ) .
            ' - '. $punkt['short_name'];
    }

    $smarty->assign('PUNKT_ODBIORU_SP_DOST_ID', PUNKT_ODBIORU_SP_DOST_ID);
    $smarty->assign('punkty_odbioru', $punkty_odbioru);
    $smarty->assign('punkty_odbioru_opcje', $punkty_odbioru_opcje);

    $output = $smarty->fetch('punkty_odbioru.tpl', $punkty_odbioru);

    return $output;
}
