<?php

 /**
 * Validate Email Addresses Via SMTP
 * This queries the SMTP server to see if the email address is accepted.
 * @copyright http://creativecommons.org/licenses/by/2.0/ - Please keep this comment intact
 * @author gabe@fijiwebdesign.com
 * @contributers adnan@barakatdesigns.net
 * @version 0.1a
 */
class SMTP_validateEmail {

 /**
  * PHP Socket resource to remote MTA
  * @var resource $sock
  */
 var $sock;

 /**
  * Current User being validated
  */
 var $user;
 /**
  * Current domain where user is being validated
  */
 var $domain;
 /**
  * List of domains to validate users on
  */
 var $domains;
 /**
  * SMTP Port
  */
 var $port = 25;
 /**
  * Maximum Connection Time to an MTA
  */
 var $max_conn_time = 30;
 /**
  * Maximum time to read from socket
  */
 var $max_read_time = 5;

 /**
  * username of sender
  */
 var $from_user = 'user';
 /**
  * Host Name of sender
  */
 var $from_domain = 'localhost';

 /**
  * Nameservers to use when make DNS query for MX entries
  * @var Array $nameservers
  */
 var $nameservers = array(
 '192.168.0.1'
);

 var $debug = false;

  /**
  * Number emails to validate
  * @var Integer $_emails_count
  */
 var $_emails_count = 0;

/**
  * Number emails to validate
  * @var Integer $_emails_count
  */
 var $_save_output = false;

 var $_validate_save = true;

 /**
  * Initializes the Class
  * @return SMTP_validateEmail Instance
  * @param $email Array[optional] List of Emails to Validate
  * @param $sender String[optional] Email of validator
  */
 function SMTP_validateEmail($emails = false, $sender = false) {
  if ($emails) {
   $this->setEmails($emails);
  }
  if ($sender) {
   $this->setSenderEmail($sender);
  }
 }

 function _parseEmail($email) {
  $parts = explode('@', $email);
 $domain = array_pop($parts);
 $user= implode('@', $parts);
 return array($user, $domain);
 }

 /**
  * Set the Emails to validate
  * @param $emails Array List of Emails
  */
 function setEmails( $emails ) {
    if( is_array($emails) ){
        $this->_emails_count += count($emails);

        foreach($emails as $email) {
            list($user, $domain) = $this->_parseEmail($email);

            if (!isset($this->domains[$domain])) {
              $this->domains[$domain] = array();
            }

            $this->domains[$domain][] = $user;
        }
    }
 }




 /**
  * Set the Email of the sender/validator
  * @param $email String
  */
 function setSenderEmail($email) {
    $parts = $this->_parseEmail($email);
    $this->from_user = $parts[0];
    $this->from_domain = $parts[1];
 }

 /**
 * Validate Email Addresses
 * @param String $emails Emails to validate (recipient emails)
 * @param String $sender Sender's Email
 * @return Array Associative List of Emails and their validation results
 */
 function validate($emails = false, $sender = false) {

    $results = array();

    if ($emails) {
      $this->setEmails($emails);
    }

    if ($sender) {
      $this->setSenderEmail($sender);
    }

    $this->progress_start( 1, count($this->domains) );

    $num = 0;

    set_time_limit(0);

    // query the MTAs on each Domain
    foreach($this->domains as $domain => $users) {
        $mxs = array();

        $this->progress_save( $num++, 'Retrieve SMTP Server via MX query on domain: ' . ' and users: ' . implode(',', $users) );

        // retrieve SMTP Server via MX query on domain
        list($hosts, $mxweights) = $this->queryMX($domain);

        // retrieve MX priorities
        for( $n = 0; $n < count($hosts); $n++){
          $mxs[$hosts[$n]] = $mxweights[$n];
        }
        asort($mxs);

        // last fallback is the original domain
        array_push($mxs, $this->domain);

        $this->debug(print_r($mxs, 1));

        $timeout = $this->max_conn_time/(count($hosts) + 1);

        // try each host
        while( list($host) = each($mxs)) {
            // connect to SMTP server
            $this->debug("try $host:$this->port\n");

            if ($this->sock = fsockopen($host, $this->port, $errno, $errstr, (float) $timeout)) {
                stream_set_timeout($this->sock, $this->max_read_time);
                break;
            }
        }

         // did we get a TCP socket
         if ( $this->sock ){
            $reply = fread($this->sock, 2082);
            $this->debug("<<<\n$reply");

            preg_match('/^([0-9]{3}) /ims', $reply, $matches);
            $code = isset($matches[1]) ? $matches[1] : '';

            if( $code != '220') {
                // MTA gave an error...
                foreach($users as $user) {
                    $results[$user.'@'.$domain] = false;
                }

                continue;
            }

            // say helo
            $this->send("HELO ".$this->from_domain);
            // tell of sender
            $this->send("MAIL FROM: <".$this->from_user.'@'.$this->from_domain.">");

            //ask for each recepient on this domain
            foreach($users as $user) {
                // ask of recepient
                $reply = $this->send("RCPT TO: <".$user.'@'.$domain.">");

                // get code and msg from response
                preg_match('/^([0-9]{3}) /ims', $reply, $matches);
                $code = isset($matches[1]) ? $matches[1] : '';

                 if ($code == '250') {
                    // you received 250 so the email address was accepted
                    $results[$user.'@'.$domain] = true;
                 } elseif ($code == '451' || $code == '452') {
                    // you received 451 so the email address was greylisted (or some temporary error occured on the MTA) - so assume is ok
                    $results[$user.'@'.$domain] = true;
                 } else {
                    $results[$user.'@'.$domain] = false;
                 }
            }

            // quit
            $this->send("quit");

            // close socket
            fclose($this->sock);
        }else{
            foreach( $users as $user ) {
                $results[$user.'@'.$domain] = false;
            }
        }
    }

    return $results;
 }

 function sav( $emails = array(), $path = '/opt/cfw/sbin/' ) {
    GLOBAL $dbh1;

    set_time_limit(0);

    $this->sav_results = $results = array();

    if ( $emails ) {
        $this->setEmails($emails);
    }

    if( $path && substr($path, -1) != '/' ){
        $path .= '/';
    }

    $this->progress_start( 1, $this->_emails_count  );

    if ( $this->_validate_save ) {
        //if( !($this->fp = fopen( TEMP_DIR . '/emails-validate-sav.txt', 'w')) ){
        //    $this->_validate_save = false;
        //}

      	$this->db = new RaksDefaultClass();
      	$this->db->table_name = 'WWW_EMAILS';
      	$this->db->gen_name = 'WWW_EMAILS_ID_GEN';
    }

    // query the MTAs on each Domain
    $num = 1;
    foreach( $this->domains as $domain => $users ) {
        foreach( $users as $user ) {
            if ( $num % 50 ) {
                flush();
            }

            $cmd_output = array(); $cmd_return_var = 0;

            $email = $user.'@'.$domain;
            $email_length = strlen($email);
            $cmd = $path . 'sav ' . $email;
            $result = null;

            $this->progress_save( $num++, $cmd);
            //echo $cmd .' ';
            $cmd_return_str = exec ( $cmd, $cmd_output, $cmd_return_var );
            //echo $cmd_return_str; echo "\n"; print_r($cmd_output);

            for ( $i = count($cmd_output) - 1; $i >= 0; $i-- ){
                if ( ($pos = stripos($cmd_output[$i], $email  )) !== false ) {
                    $this->sav_results[$email] = $cmd_output[$i];

                    if ( stripos($cmd_output[$i], 'succeeded' , $pos + $email_length ) !== false ) {
                        $result = true;
                    } else if ( stripos($cmd_output[$i], 'failed', $pos + $email_length ) !== false ) {
                        $result = false;
                    }

                    break;
                }
            }

            $results[$email] = $result;

            echo $email . ":" . (is_null($result) ? 'null' : ( $result ? 'true' : 'false' )) . "\n";

            if ( $this->_validate_save  ) {
                //fwrite( $this->fp, $email . ":" . (is_null($result) ? 'null' : ( $result ? 'true' : 'false' )) . "\n" );
                $this->db->clear_errors();
                $rekord = array(
                    'ID' => 0,
                    'EMAIL' => $email,
                    'RESULT' => $result,
                    'MSG' => implode("\n", $cmd_output),
                    'ZGODA' => 1,
                );
                $this->db->set_values($rekord);

                if( $this->_validate_save == 'update' ){
                    if ( !$this->db->db_update( array( 'RESULT', 'MSG' ), array( 'where' => " email = '$email' " ) ) ){

                    }
                }else if ( !$this->db->db_insert() ){

                }
            }
        }
    }

    if(  $this->_validate_save  ){
        //fclose($this->fp);
    }

    return $results;
 }


 function send($msg) {
  fwrite($this->sock, $msg."\r\n");

  $reply = fread($this->sock, 2082);

  $this->debug(">>>\n$msg\n");
  $this->debug("<<<\n$reply");

  return $reply;
 }

 /**
  * Query DNS server for MX entries
  * @return
  */
 function queryMX($domain) {
  $hosts = array();
 $mxweights = array();
  if (function_exists('getmxrr')) {
   getmxrr($domain, $hosts, $mxweights);
  } else {
   // windows, we need Net_DNS
  require_once 'Net/DNS.php';

  $resolver = new Net_DNS_Resolver();
  $resolver->debug = $this->debug;
  // nameservers to query
  $resolver->nameservers = $this->nameservers;
  $resp = $resolver->query($domain, 'MX');
  if ($resp) {
   foreach($resp->answer as $answer) {
    $hosts[] = $answer->exchange;
    $mxweights[] = $answer->preference;
   }
  }

  }
 return array($hosts, $mxweights);
 }

 /**
  * Simple function to replicate PHP 5 behaviour. http://php.net/microtime
  */
 function microtime_float() {
  list($usec, $sec) = explode(" ", microtime());
  return ((float)$usec + (float)$sec);
 }

 function debug($str) {
  if ($this->debug) {
   echo htmlentities($str);
  }
 }

 function progress_start( $action_cnt = 1, $record_numbers = 0 ) {

    if ( empty($this->progress_file) ) {
        if ( defined('TEMP_DIR') && file_exists(TEMP_DIR) ) {

            if( file_exists(TEMP_DIR . '/progress' )  ){
                $tmp_dir = TEMP_DIR . '/progress';
            }else if( @mkdir(TEMP_DIR . '/progress', 0777) ){
                $tmp_dir = TEMP_DIR . '/progress';

                chmod( TEMP_DIR . '/progress', 0777);
            }else{
                $tmp_dir = TEMP_DIR;
            }
        } else {
            $tmp_dir = sys_get_temp_dir();
        }

        $this->progress_file = tempnam($tmp_dir, 'progress');

        if ( $fp = fopen($this->progress_file, 'w') ) {
            fclose($fp);

            chmod( $this->progress_file, 0666);
        }else{
            $this->progress_file = false;
        }
    }

    $this->action_cnt = $action_cnt;
    $this->action_num = 1;
    $this->loop_start_time = $this->progress_start_time = fmicrotime();

    $this->record_numbers = $record_numbers;
 }

 function progress_end() {
    if ( !empty($this->progress_file) && file_exists($this->progress_file) ) {
        unlink( $this->progress_file );
    }
 }

 function progress_next( $record_numbers = 0 ) {

    $this->action_num++ ;
    //na wszelki wypadek
    if( $this->action_num > $this->action_cnt ) $this->action_cnt = $this->action_num;

    $this->loop_start_time = $this->progress_start_time = fmicrotime();

    $this->record_numbers = $record_numbers;
 }

 function progress_save( $record, $action ) {

    if ( empty($this->progress_file) ) return false;

    $loop_end_time = fmicrotime();

    $records_left = $this->record_numbers - $record;
    $loop_time = ($loop_end_time - $this->loop_start_time );

    $pass_time = $loop_end_time - $this->progress_start_time;
    $avg_loop_time = ( $record != 0 ) ? ($pass_time / $record) : $loop_time;


    $fstr  = '';
    $fstr .= 'var record_number = '.intval($record).'; '."\n";
    $fstr .= 'var record_numbers = '.intval($this->record_numbers).'; '."\n";
    $fstr .= 'var procent = '.round(($record / $this->record_numbers)*100, 2).'; '."\n";
    $fstr .= 'var loop_time = '.floatval($loop_time).'; '."\n";
    $fstr .= 'var left_time = '.round(floatval($avg_loop_time * $records_left), 2).'; '."\n";
    $fstr .= 'var pass_time = '.round(floatval( $pass_time ), 2).'; '."\n";

    $fstr .= 'var action_name= \''.addcslashes($action, "\\'") .'\'; '."\n";
    $fstr .= 'var action_num = '.$this->action_num.'; '."\n";
    $fstr .= 'var action_cnt = '.$this->action_cnt.'; '."\n";

    if ( $fid = fopen( $this->progress_file, 'w') ){
        fwrite($fid, $fstr);
        fclose( $fid );
    }

    $this->loop_start_time = $loop_end_time;
 }

}


?>
