/**
 * PortableDrag.js:
 * beginDrag() is designed to be called from an onmousedown event handler.
 * elementResize may be the element that received the mousedown event, or it
 * may be some containing element.  event must be the Event object for the
 * mousedown event.  This implementation works with both the DOM Level 2
 * event model and the IE Event model.
 **/

function startResize(resizeElement, event, minWidth){

    if( !minWidth < 0 ){
        minWidth = 0;
    }
    
    //Compute the distance between the upper-left corner of the element
    //and the mouse click. The resMoveHandler function below needs these values.
    var sasiadl = resizeElement.previousSibling;
    while( sasiadl && sasiadl.nodeName != 'DIV' ){
        sasiadl = sasiadl.previousSibling;
    }
    
    
    if( !sasiadl ){
        alert('Nie znaleziono elementu DIV z lewej strony!!');
        return false;
    }
    
    /**************************************************************************/
    var sasiadp = resizeElement.nextSibling;
    while(sasiadp && sasiadp.nodeName != 'DIV' ){
        sasiadp = sasiadp.nextSibling;
    }

    if(!sasiadp){
        alert('Nie znaleziono elementu DIV z prawej strony!!');
        return false;
    }

    

    var startX     = event.clientX;
    var startLeft  = parseInt(resizeElement.offsetLeft);
    var startWidth = parseInt(sasiadl.clientWidth);
    //maksymalna szeroko elementu, teoretycznie
    maxWidth = (sasiadl.parentNode.clientWidth - sasiadl.offsetLeft - resizeElement.offsetWidth);
    
    //sasiadl.style.overflow = 'hidden';
    //sasiadl.style.width    = startWidth + 'px';
    //sasiadl.style.display  = 'block';


    //window.status = 'x = ' + event.clientX + ', y = ' + event.clientY + ' L = ' + startWidthL + ' P: ' + startWidthP;
    //var deltaY = event.clientY - parseInt(resizeElement.style.top);

    // Register the event handlers that will respond to the mousemove events
    // and the mouseup event that follow this mousedown event.
    if (document.addEventListener) {  // DOM Level 2 Event Model
        // Register capturing event handlers
        document.addEventListener("mousemove", resMoveHandler, true);
        document.addEventListener("mouseup", resUpHandler, true);
    }
    else if (document.attachEvent) {  // IE 5+ Event Model
        // In the IE Event model, we can't capture events, so these handlers
        // are triggered when only if the event bubbles up to them.
        // This assumes that there aren't any intervening elements that
        // handle the events and stop them from bubbling.
        document.attachEvent("onmousemove", resMoveHandler);
        document.attachEvent("onmouseup", resUpHandler);
    }
    else {                            // IE 4 Event Model
        // In IE 4 we can't use attachEvent(), so assign the event handlers
        // directly after storing any previously assigned handlers so they
        // can be restored.  Note that this also relies on event bubbling.
        var oldmovehandler = document.onmousemove;
        var olduphandler = document.onmouseup;
        document.onmousemove = resMoveHandler;
        document.onmouseup = resUpHandler;
    }

    // We've handled this event.  Don't let anybody else see it.
    if (event.stopPropagation) event.stopPropagation();   // DOM Level 2
    else event.cancelBubble = true;                       // IE

    // Now prevent any default action.
    if (event.preventDefault) event.preventDefault();     // DOM Level 2
    else event.returnValue = false;                       // IE

  /**
  * This is the handler that captures mousemove events when an element
  * is being dragged.  It is responsible for moving the element.
  **/
  function resMoveHandler(e) {
  	if (!e) e = window.event;  // IE event model

    // Move the element to the current mouse position, adjusted as
  	// necessary by the offset of the initial mouse click.
  	var deltaX   = (e.clientX - startX);
  	var newLeft  = startLeft  + deltaX;
  	var newWidth = startWidth + deltaX;
  	
    //window.status =  "clientX = " + e.clientX + "startX  = " + startX + 'newWidth = ' + newWidth;

  	if(newWidth < minWidth){
      resizeElement.style.left = (sasiadl.offsetLeft + minWidth) + 'px';
      //resizeElement.style.visibility = 'visible';
    }else if( newWidth > maxWidth ){
      resizeElement.style.left = (sasiadl.parentNode.clientWidth - resizeElement.clientWidth) + 'px';
      //resizeElement.style.visibility = 'hidden';
    }else{
      resizeElement.style.left = (newLeft) + 'px';
      //resizeElement.style.visibility = 'visible';
    }
    
  	//resizeElement.style.left = (e.clientX - deltaX) + "px";
  	//resizeElement.style.top = (e.clientY - deltaY) + "px";

  	// And don't let anyone else see this event.
  	if (e.stopPropagation)
      e.stopPropagation();       // DOM Level 2
  	else
      e.cancelBubble = true;                       // IE
  }

  /**
     * This is the handler that captures the final mouseup event that
     * occurs at the end of a drag.
  **/
  function resUpHandler(e) {
  	if (!e) e = window.event;  // IE event model

  	var deltaX   = (e.clientX - startX);
  	var newLeft  = startLeft + deltaX;
  	var newWidth = startWidth + deltaX;

  	if( deltaX != 0 ){
        if( sasiadl.clientWidth == 0 ){
            sasiadl.style.display = 'block';
        }
      	//window.status =  "clientX = " + e.clientX + "startX  = " + startX + 'newWidth = ' + newWidth;
      	if(newWidth < minWidth){
          sasiadl.style.width = minWidth + 'px';
          sasiadp.style.marginLeft = (minWidth + resizeElement.clientWidth) + 'px';
          resizeElement.style.left = (sasiadl.offsetLeft + minWidth) + 'px';
          //resizeElement.style.visibility = 'visible';
        }else if( newWidth > maxWidth ){
          sasiadl.style.width = maxWidth + 'px';
          sasiadp.style.marginLeft = sasiadl.parentNode.clientWidth + 'px';
          resizeElement.style.left = (sasiadl.parentNode.clientWidth - resizeElement.clientWidth) + 'px';
          //resizeElement.style.visibility = 'hidden';
        }else{

          sasiadl.style.width = newWidth + 'px';
          sasiadp.style.marginLeft = (newLeft + resizeElement.clientWidth) + 'px';
          resizeElement.style.left = (newLeft) + 'px';

          //resizeElement.style.visibility = 'visible';
        }

    }else{
        showHideLeftPanel();
    }

    

  	// Unregister the capturing event handlers.
  	if (document.removeEventListener) {    // DOM Event Model
  	    document.removeEventListener("mouseup", resUpHandler, true);
  	    document.removeEventListener("mousemove", resMoveHandler, true);
  	}
  	else if (document.detachEvent) {       // IE 5+ Event Model
  	    document.detachEvent("onmouseup", resUpHandler);
  	    document.detachEvent("onmousemove", resMoveHandler);
  	}
  	else {                                 // IE 4 Event Model
  	    document.onmouseup = olduphandler;
  	    document.onmousemove = oldmovehandler;
  	}

  	// And don't let the event propagate any further.
  	if (e.stopPropagation) e.stopPropagation();       // DOM Level 2
  	else e.cancelBubble = true;                       // IE
  }
  
  
  
  function showHideLeftPanel(){

      if( sasiadl.clientWidth > 0 ){
          resizeElement.sasiadl_old_width = sasiadl.clientWidth;
          sasiadl.style.display = 'none';
          resizeElement.style.left = '0px';
          sasiadp.style.marginLeft =  resizeElement.clientWidth + 'px';
      }else{
          sasiadl.style.display = 'block';
          sasiadl.style.width   = resizeElement.sasiadl_old_width + 'px';
          resizeElement.style.left = (resizeElement.sasiadl_old_width - 1) + 'px';
          sasiadp.style.marginLeft =  (resizeElement.sasiadl_old_width + 1) + 'px';
      }

  }
}


