
/******************************************************************************/
/****                                Tables                                ****/
/******************************************************************************/


CREATE GENERATOR WWW_PACZKI_ID_GEN;

CREATE TABLE WWW_PACZKI (
    ID                  ID_D NOT NULL /* ID_D = INTEGER DEFAULT 0 NOT NULL */,
    KURIER              STRING10_D NOT NULL /* STRING10_D = VARCHAR(10) */,
    DOC_TYPE            STRING10_D /* STRING10_D = VARCHAR(10) */,
    DOC_ID              SLOWNIK_D /* SLOWNIK_D = INTEGER */,
    NUMER               STRING50_D NOT NULL /* STRING50_D = VARCHAR(50) */,
    STATUS              STRING100_D NOT NULL /* STRING100_D = VARCHAR(100) */,
    KURIER_DOKUMENT     STRING50_D /* STRING50_D = VARCHAR(50) */,
    CONTACT_ID          SLOWNIK_D /* SLOWNIK_D = INTEGER */,
    CONTACT_SHORT_NAME  STRING50_D /* STRING50_D = VARCHAR(50) */,
    CONTACT_FULL_NAME   STRING250_D /* STRING250_D = VARCHAR(250) */,
    CONTACT_PLACE       STRING50_D /* STRING50_D = VARCHAR(50) */,
    CONTACT_POST_CODE   STRING10_D /* STRING10_D = VARCHAR(10) */,
    CONTACT_STREET      STRING50_D /* STRING50_D = VARCHAR(50) */,
    CONTACT_HOUSE       STRING10_D /* STRING10_D = VARCHAR(10) */,
    CONTACT_LOCAL       STRING10_D /* STRING10_D = VARCHAR(10) */,
    CONTACT_PHONE       STRING20_D /* STRING20_D = VARCHAR(20) */,
    CONTACT_EMAIL       STRING60_D /* STRING60_D = VARCHAR(60) */,
    CDATE               DATA_TIME_D NOT NULL /* DATA_TIME_D = TIMESTAMP */,
    MDATE               DATA_TIME_D NOT NULL /* DATA_TIME_D = TIMESTAMP */
);




/******************************************************************************/
/****                             Primary Keys                             ****/
/******************************************************************************/

ALTER TABLE WWW_PACZKI ADD CONSTRAINT WWW_PACZKI_ID_PK PRIMARY KEY (ID);


/******************************************************************************/
/****                               Indices                                ****/
/******************************************************************************/

CREATE INDEX WWW_PACZKI_CONTACT_EMAIL_IND ON WWW_PACZKI (CONTACT_EMAIL);
CREATE INDEX WWW_PACZKI_CONTACT_FULL_IND ON WWW_PACZKI (CONTACT_FULL_NAME);
CREATE INDEX WWW_PACZKI_CONTACT_ID_IND ON WWW_PACZKI (CONTACT_ID);
CREATE INDEX WWW_PACZKI_CONTACT_SHROT_IND ON WWW_PACZKI (CONTACT_SHORT_NAME);
CREATE INDEX WWW_PACZKI_DATES_IND ON WWW_PACZKI (CDATE, MDATE);
CREATE INDEX WWW_PACZKI_DOC_IND ON WWW_PACZKI (DOC_ID, DOC_TYPE);
CREATE INDEX WWW_PACZKI_KURIER_DOKUMENT_IND ON WWW_PACZKI (KURIER_DOKUMENT);
CREATE INDEX WWW_PACZKI_KURIER_IND ON WWW_PACZKI (KURIER);
CREATE INDEX WWW_PACZKI_NUMER_IND ON WWW_PACZKI (NUMER);


/******************************************************************************/
/****                               Triggers                               ****/
/******************************************************************************/


SET TERM ^ ;


/******************************************************************************/
/****                         Triggers for tables                          ****/
/******************************************************************************/



/* Trigger: WWW_PACZKI_BI */
CREATE TRIGGER WWW_PACZKI_BI FOR WWW_PACZKI
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL) THEN
    NEW.ID = GEN_ID(WWW_PACZKI_ID_GEN,1);
END
^


SET TERM ; ^



/******************************************************************************/
/****                              Privileges                              ****/
/******************************************************************************/


/* Privileges of users */
GRANT ALL ON WWW_PACZKI TO RAKS3000 WITH GRANT OPTION;

/* Privileges of roles */
GRANT ALL ON WWW_PACZKI TO WWW_ROLE;
